@@ -709,10 +709,10 @@ class J2534Connection(BaseConnection):
     :type name: string
     :param debug: This will enable windows debugging mode in the dll (see tactrix doc for additional information)
     :type debug: boolean
-    :param args: Optional parameters list (Unused right now).
-    :type args: list
-    :param kwargs: Optional parameters dictionary Unused right now).
-    :type kwargs: dict
+    :param protocol: CAN protocol
+    :type protocol: Protocol_ID
+    :param baudrate: Operation bauderate
+    :type baudrate: int
 
     """
 
@@ -727,27 +727,36 @@ class J2534Connection(BaseConnection):
     exit_requested: bool
     opened: bool
 
-    def __init__(self, windll: str, rxid: int, txid: int, name: Optional[str] = None, debug: bool = False, *args, **kwargs):
+    def __init__(self,
+                 windll: str,
+                 rxid: int,
+                 txid: int,
+                 name: Optional[str] = None,
+                 debug: bool = False,
+                 protocol = None,
+                 baudrate = 500000,
+                 ):
         BaseConnection.__init__(self, name)
 
-        # Set up a J2534 interface using the DLL provided
-        try:
-            self.interface = J2534(windll=windll, rxid=rxid, txid=txid)
-        except FileNotFoundError:
-            raise RuntimeError('DLL not found')
-
-        # Set the protocol to ISO15765, Baud rate to 500000
-        self.protocol = Protocol_ID.ISO15765
-        self.baudrate = 500000
+        self.protocol = protocol if protocol else Protocol_ID.ISO15765
+        self.baudrate = baudrate
         self.debug = debug
 
         try:
+            # Set up a J2534 interface using the DLL provided
+            self.interface = J2534(windll=windll, rxid=rxid, txid=txid)
+
             # Open the interface (connect to the DLL)
             self.result, self.devID = self.interface.PassThruOpen()
+        except FileNotFoundError:
+            raise RuntimeError('DLL not found')
         except OSError as e:
             if e.errno in [0x16, 0xe06d7363]:
                 raise RuntimeError('J2534 Device busy')
-            raise RuntimeError('%s, %X' % (type(e).__name__, e.errno))
+            exception_str = type(e).__name__
+            if e.errno is not None:
+                exception_str += ', %s' % e.errno
+            raise RuntimeError(exception_str)
 
         self.log_last_operation("PassThruOpen", with_raise=True)
 
@@ -768,7 +777,7 @@ class J2534Connection(BaseConnection):
         self.log_last_operation("PassThruConnect", with_raise=True)
 
         configs = SCONFIG_LIST([
-            (Ioctl_ID.DATA_RATE.value, 500000),
+            (Ioctl_ID.DATA_RATE.value, self.baudrate),
             (Ioctl_ID.LOOPBACK.value, 0),
             (Ioctl_ID.ISO15765_BS.value, 0x20),
             (Ioctl_ID.ISO15765_STMIN.value, 0),
@@ -795,6 +804,7 @@ class J2534Connection(BaseConnection):
 
     def open(self) -> "J2534Connection":
         self.exit_requested = False
+        self.sem = threading.Semaphore()
         self.rxthread = threading.Thread(target=self.rxthread_task, daemon=True)
         self.rxthread.start()
         self.opened = True
@@ -812,6 +822,7 @@ class J2534Connection(BaseConnection):
 
     def rxthread_task(self) -> None:
         while not self.exit_requested:
+            self.sem.acquire()
             try:
                 result, data, numMessages = self.interface.PassThruReadMsgs(self.channelID, self.protocol.value, 1, 1)
                 if data is not None:
@@ -819,6 +830,8 @@ class J2534Connection(BaseConnection):
             except Exception:
                 self.logger.critical("Exiting J2534 rx thread")
                 self.exit_requested = True
+            self.sem.release()
+            time.sleep(0.001)
 
     def log_last_operation(self, exec_method: str, with_raise = False) -> None:
         if self.result != Error_ID.ERR_SUCCESS:
@@ -844,9 +857,13 @@ class J2534Connection(BaseConnection):
         self.log_last_operation('PassThruClose')
 
     def specific_send(self, payload: bytes, timeout: Optional[float] = None):
-        if timeout is None:
-            timeout = 0
-        result = self.interface.PassThruWriteMsgs(self.channelID, payload, self.protocol.value, Timeout=int(timeout * 1000))
+        timeout = 0 if timeout is None else timeout
+
+        # Fix for avoid ERR_CONCURRENT_API_CALL. Stop reading
+        self.sem.acquire()
+        self.result = self.interface.PassThruWriteMsgs(self.channelID, payload, self.protocol.value, Timeout=int(timeout * 1000))
+        self.log_last_operation('PassThruWriteMsgs', with_raise=True)
+        self.sem.release()
 
     def specific_wait_frame(self, timeout: Optional[float] = None) -> Optional[bytes]:
         if not self.opened:
